//+------------------------------------------------------------------+
//|                                                   HeikenAshi.mq4 |
//|                                            Copyright 2021, ernst |
//|                             https://www.mql5.com/en/users/pippod |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, ernst"
#property link      "https://www.mql5.com/en/users/pippod"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 6
#property indicator_plots   6
//--- plot High
#property indicator_label1  "HA High"
//--- plot Low
#property indicator_label2  "HA Low"
//--- plot Open
#property indicator_label3  "HA Open"
//--- plot Close
#property indicator_label4  "HA Close"
//--- input parameters
input color    BullColor=clrDodgerBlue;
input color    BearColor=clrFireBrick;
//--- indicator buffers
double         OpenBuffer[];
double         HighBuffer[];
double         LowBuffer[];
double         CloseBuffer[];
double         HiLoBuffer[];
double         LoHiBuffer[];
//---
int scale;
//---
#include <Charts\Chart.mqh>
CChart chrt;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,HighBuffer);
   ArraySetAsSeries(HighBuffer,false);
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1,clrNONE);
   SetIndexBuffer(1,LowBuffer);
   ArraySetAsSeries(LowBuffer,false);
   SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,1,clrNONE);
   SetIndexBuffer(2,OpenBuffer);
   ArraySetAsSeries(OpenBuffer,false);
   SetIndexStyle(2,DRAW_HISTOGRAM,EMPTY,EMPTY,BearColor);
   SetIndexBuffer(3,CloseBuffer);
   ArraySetAsSeries(CloseBuffer,false);
   SetIndexStyle(3,DRAW_HISTOGRAM,EMPTY,EMPTY,BullColor);
   SetIndexBuffer(4,HiLoBuffer);
   ArraySetAsSeries(HiLoBuffer,false);
   SetIndexStyle(4,DRAW_HISTOGRAM,STYLE_SOLID,1,BearColor);
   SetIndexLabel(4,NULL);
   SetIndexBuffer(5,LoHiBuffer);
   ArraySetAsSeries(LoHiBuffer,false);
   SetIndexStyle(5,DRAW_HISTOGRAM,STYLE_SOLID,1,BullColor);
   SetIndexLabel(5,NULL);
//---
   chrt.Attach(ChartID());
   chrt.Mode(CHART_LINE);
   chrt.ColorChartLine(clrNONE);
//---
   scale=(int)chrt.GetInteger(CHART_SCALE);
   PlotSetWidth();
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   ArraySetAsSeries(open,false);
   ArraySetAsSeries(high,false);
   ArraySetAsSeries(low,false);
   ArraySetAsSeries(close,false);
//---
   double haOpen, haHigh, haLow, haClose;
//---
   int begin=(rates_total!=prev_calculated)?prev_calculated:prev_calculated-1;
//---
   if(!prev_calculated)
     {
      if(close[0]>open[0])
        {
         LoHiBuffer[0]=LowBuffer[0]=low[0];
         HiLoBuffer[0]=HighBuffer[0]=high[0];
        }
      else
        {
         HiLoBuffer[0]=HighBuffer[0]=high[0];
         LoHiBuffer[0]=LowBuffer[0]=low[0];
        }
      OpenBuffer[0]=open[0];
      CloseBuffer[0]=close[0];
      begin++;
     }
//---
   for(int i=begin;i<rates_total && !_StopFlag;i++)
     {
      OpenBuffer[i]=haOpen=(OpenBuffer[i-1]+CloseBuffer[i-1])/2;
      CloseBuffer[i]=haClose=(open[i]+high[i]+low[i]+close[i])/4;
      HighBuffer[i]=haHigh=MathMax(high[i],MathMax(haOpen,haClose));
      LowBuffer[i]=haLow=MathMin(low[i],MathMin(haOpen,haClose));
//---
      if(haClose<haOpen)
        {
         LoHiBuffer[i]=haLow;
         HiLoBuffer[i]=haHigh;
        }
      else
        {
         LoHiBuffer[i]=haHigh;
         HiLoBuffer[i]=haLow;
        }
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   chrt.Detach();
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   if(id==CHARTEVENT_CHART_CHANGE && scale!=(int)chrt.GetInteger(CHART_SCALE))
     {
      scale=(int)chrt.GetInteger(CHART_SCALE);
      PlotSetWidth();
     }   
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void PlotSetWidth()
  {
//---
   switch(scale)
     {
      case 0:
      case 1:
         PlotWidth(1);
         return;
      case 2:   
         PlotWidth(2);
         return;
      case 3:   
         PlotWidth(3);
         return;
      case 4:   
         PlotWidth(7);
         return;
      case 5:   
         PlotWidth(13);
         return;
     }
//---
  }  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void PlotWidth(int width)
  {
   SetIndexStyle(2,EMPTY,EMPTY,width);
   SetIndexStyle(3,EMPTY,EMPTY,width);
//---
  }
//+------------------------------------------------------------------+
